﻿using Microsoft.Extensions.Logging;
using BeSafePlus.ViewModels;
using BeSafePlus.Database;
using SQLitePCL;
using BeSafePlus.Sensors;
using BeSafePlus.Services.Interfaces;
using BeSafePlus.Services;
using SkiaSharp.Views.Maui.Controls.Hosting;
using ZXing.Net.Maui.Controls;
using BeSafePlus.Views;
using BeSafePlus.Clients;



// Android specific inclusions
#if ANDROID
using BeSafePlus.Platforms.Android.AndroidSensor;
using BeSafePlus.Platforms.Android.Scheduling;
using BeSafePlus.Platforms.Android.UI;
#endif

// iOS specific inclusions
#if IOS
using BeSafePlus.Platforms.iOS.Sensor;
using BeSafePlus.Platforms.iOS.UI;
#endif

namespace BeSafePlus;

public static class MauiProgram
{
	public static MauiApp CreateMauiApp()
	{

        //SW: Added this to ensure that the app does not crash when an unhandled exception occurs.
        AppDomain.CurrentDomain.UnhandledException += (sender, e) =>
        {
            Console.WriteLine($"Unhandled Exception: {e.ExceptionObject}");
            // Log, report, or handle the exception appropriately.

            //SW: PLEASE ADD A NOTIFICAITON HERE FOR RESEARCH PURPOSES - SO THAT 
            //YOU KNOW THAT THIS HAPPEND:

        };

        TaskScheduler.UnobservedTaskException += (sender, e) =>
        {
            Console.WriteLine($"Unobserved Task Exception: {e.Exception}");
            e.SetObserved(); // Prevents application crash

            //SW: PLEASE ADD A NOTIFICAITON HERE FOR RESEARCH PURPOSES - SO THAT 
            //YOU KNOW THAT THIS HAPPEND:

        };

        //SW: I added this android specific unhandled exception handler
#if ANDROID
        Android.Runtime.AndroidEnvironment.UnhandledExceptionRaiser += (sender, e) =>
        {
            Console.WriteLine($"Android Unhandled Exception: {e.Exception}");
            e.Handled = true; // Prevents app from crashing

            //SW: PLEASE ADD A NOTIFICAITON HERE FOR RESEARCH PURPOSES - SO THAT 
            //YOU KNOW THAT THIS HAPPEND:
        };
#endif

        try
        {
            // Initialize SQLite's native libraries 
#if IOS
        Batteries_V2.Init();
#endif
        }
        catch (Exception ex)
        {
            Console.WriteLine($"Error initializing SQLite: {ex.Message}");
        }
       
        var builder = MauiApp.CreateBuilder();
        builder.UseMauiApp<App>().UseBarcodeReader().UseSkiaSharp().ConfigureFonts(fonts =>
        {
            fonts.AddFont("OpenSans-Regular.ttf", "OpenSansRegular");
            fonts.AddFont("OpenSans-Semibold.ttf", "OpenSansSemibold");
        });
        builder.Services.AddLogging();

#if DEBUG
        builder.Logging.AddDebug();
#endif

#if ANDROID
        builder.Services.AddSingleton<IActivityTrackingSensor, ActivityTrackingSensor>()
                        .AddSingleton<IScheduleService, ScheduleService>()
                        .AddSingleton<IUIService, UIService>();
#endif

#if IOS
        builder.Services.AddSingleton<IActivityTrackingSensor, BeSafePlus.Platforms.iOS.Sensor.ActivityTrackingSensor>()
                        .AddSingleton<IUIService, UIService>();;
#endif

        builder.Services.AddSingleton<SensorEventHandler>();
        // firebase 
        //builder.Services.AddSingleton(new FirebaseClient("https://besafeplus-1d01d-default-rtdb.europe-west1.firebasedatabase.app/"));

        /* Pages */
        // Main page and vm should be singleton
        builder.Services.AddSingleton<MainViewModel>();
        builder.Services.AddSingleton<MainPage>();

        // Other pages and vm should be transient
        builder.Services
            .AddTransient<ActivityHistoryViewModel>()
            .AddTransient<ProfileViewModel>()
            .AddTransient<SettingsViewModel>();

        builder.Services
            .AddTransient<ActivityHistoryPage>()
            .AddTransient<ProfilePage>()
            .AddTransient<SettingsPage>()
            .AddTransient<QrScannerPage>();

        /* others */
        builder.Services.AddSingleton<ILocalDatabase, LocalDatabase>();
       // builder.Services.AddSingleton<SensorEventHandler>(); ??
        builder.Services.AddSingleton<IReportService, DailyReportService>();
        builder.Services.AddSingleton<IRedCapClient, RedCapClient>();

        return builder.Build();
	}
}
