﻿using Android.App;
using Android.Content;
using Android.OS;
using Android.Util;

namespace BeSafePlus.Platforms.Android.Scheduling
{
    public class ScheduleService : IScheduleService
    {
        public void ScheduleAlarm<TReceiver>(int requestCode, TimeSpan timeSpan)
            where TReceiver : BroadcastReceiver
        {
            try
            {
                var context = global::Android.App.Application.Context;

                if (context.GetSystemService(Context.AlarmService) is not AlarmManager alarmManager)
                {
                    return;
                }
                var intent = new Intent(context, typeof(TReceiver));

                var flags = PendingIntentFlags.UpdateCurrent;
                if (Build.VERSION.SdkInt >= BuildVersionCodes.S) // API 31+
                {
                    flags |= PendingIntentFlags.Mutable;
                }

                var pendingIntent = PendingIntent.GetBroadcast(context, requestCode, intent, flags);

                var midnightMillis = DateTimeOffset.Now.ToUnixTimeMilliseconds() + (long)timeSpan.TotalMilliseconds;

                if (pendingIntent != null)
                {
                    // If exact timing permissions are allowed
                    if (alarmManager.CanScheduleExactAlarms())
                    {
                        alarmManager.SetExactAndAllowWhileIdle(AlarmType.RtcWakeup, midnightMillis, pendingIntent);
                    }
                    else
                    {
                        // Use inexact
                        alarmManager.SetAndAllowWhileIdle(AlarmType.RtcWakeup, midnightMillis, pendingIntent);
                        // or invoke event for user to allow it
                    }
                }
            }
            catch (Exception e)
            {
                Log.Error("ScheduleService", $"Failed to schedule alarm for {typeof(TReceiver)}, with error: {e.Message}");
                throw;
            }
        }
    }
}
