﻿using MQTTnet;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BeSafePlus
{
    public class MQTTLog
    {
        private const string DefaultTopic = "besafeau/flare/log";
        public IMqttClient? _mqttClient;
        public async Task Setup()
        {
            var factory = new MqttClientFactory();
            _mqttClient = factory.CreateMqttClient();
            var options = new MqttClientOptionsBuilder()
                .WithTcpServer("broker.hivemq.com")
                .Build();
            try
            {
                await _mqttClient.ConnectAsync(options);
                Console.WriteLine("Connected to MQTT broker in background.");
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Error connecting to MQTT broker in background: {ex.Message}");
            }
        }
        public async Task DisconnectAsync()
        {
            if (_mqttClient != null && _mqttClient.IsConnected)
            {
                await _mqttClient.DisconnectAsync();
                Console.WriteLine("Disconnected from MQTT broker in background.");
            }
        }
        public async Task PublishMessageAsync(string payload, string topic = DefaultTopic)
        {
            if (_mqttClient == null || !_mqttClient.IsConnected)
            {
                Console.WriteLine("MQTT client is not connected in background.");
                return;
            }
            var message = new MqttApplicationMessageBuilder()
                .WithTopic(topic)
                .WithPayload(payload)
                .Build();
            await _mqttClient.PublishAsync(message);
            Console.WriteLine($"Published message in background to topic {topic}: {payload}");
        }
    }
}
