using BeSafePlus.ViewModels;
using Microsoft.Maui.Controls;
using Microcharts.Maui;
using SkiaSharp;
using System.Collections.Generic;
using Microsoft.Maui.Controls;

namespace BeSafePlus
{
    public partial class ActivityHistoryPage : ContentPage
    {
        private ActivityHistoryViewModel _viewModel;

        private static readonly string[] DaysOfWeek = { "Mandag", "Tirsdag", "Onsdag", "Torsdag", "Fredag", "Lørdag", "Søndag" };
        public ActivityHistoryPage(ActivityHistoryViewModel vm)
        {
            InitializeComponent();

            BindingContext = _viewModel = vm;

            _viewModel.PropertyChanged += (sender, e) =>
            {
                if (e.PropertyName == nameof(ActivityHistoryViewModel.WeeklyStepData) ||
                    e.PropertyName == nameof(ActivityHistoryViewModel.StepGoal) ||
                    e.PropertyName == nameof(ActivityHistoryViewModel.TotalStepsThisWeek) ||
                    e.PropertyName == nameof(ActivityHistoryViewModel.AccumulatedSteps))
                {
                    LoadChart();
                }
            };

            LoadChart();
        }
        protected override async void OnAppearing()
        {
            base.OnAppearing();

            await _viewModel.LoadStepGoal();

            await _viewModel.LoadWeeklySteps();

            _viewModel.AccumulatedSteps = await _viewModel.GetStepCount();

            LoadChart();
        }
        private async void LoadChart()
        {
            if (_viewModel?.WeeklyStepData == null || !_viewModel.WeeklyStepData.Any()) return;

            var entries = new List<Microcharts.ChartEntry>();

            int stepGoal = await _viewModel.GetStepGoal();

            DateTime today = DateTime.Now.Date;

            foreach (var stepData in _viewModel.WeeklyStepData)
            {
                if (DateTime.TryParse(stepData.Date, out DateTime stepDate))
                {
                    int dayIndex = (int) stepDate.DayOfWeek;

                    string dayName = DaysOfWeek[dayIndex == 0 ? 6 : dayIndex - 1];

                    float percentage = (float)stepData.Steps / stepGoal;

                    var color = GetStepColor(stepData.Steps, stepGoal);

                    float barHeight = stepData.Steps == 0 ? 0 : percentage * stepGoal;

                    bool isToday = stepDate.Date == today;

                    entries.Add(new Microcharts.ChartEntry(barHeight)
                    {
                        Label = isToday ? $"*{dayName}" : dayName,
                        ValueLabel = $"{stepData.Steps} ({percentage * 100:0}%)",
                        Color = color,
                        ValueLabelColor = SKColors.Black,
                        TextColor = SKColors.Black
                    });
                }
            }

            BarChart.Chart = new Microcharts.BarChart
            {
                Entries = entries,
                BackgroundColor = SKColors.White,
                LabelTextSize = 40,
                ValueLabelTextSize = 40,
                LabelColor = SKColors.Black,
                LabelOrientation = Microcharts.Orientation.Horizontal,
                MaxValue = stepGoal
            };
        }
        private SKColor GetStepColor(int steps, int stepGoal)
        {
            if (steps == 0) return SKColors.White;

            float percentage = (float) steps / stepGoal;

            return percentage switch
            {
                < 0.2f => SKColor.Parse("#FF0000"), // Rød
                < 0.4f => SKColor.Parse("#FF8000"), // Orange
                < 0.6f => SKColor.Parse("#FFFF00"), // Gul
                < 0.8f => SKColor.Parse("#00FF00"), // Lysegrøn
                _ => SKColor.Parse("#008000"), // Grøn
            };
        }
    }
}